﻿using UnityEngine;
using System.Collections;
using System;
using System.Linq;

    /// <summary>
    /// One of these should be placed on every card gameObject.
    /// </summary>

    [Serializable]
    public struct CardDesc
    {
        //public int id;
        public int color;
        public int rarity;
        public int power;
        public string action; // 16 different
        public Sprite sprite;
    };

    public class Card : MonoBehaviour
    {
        public CardDesc desc;

        private Deck deck;
        public Deck Deck
        {
            get
            {
                return deck;
            }
            set
            {
                if (deck != value)
                {
                    if (deck != null)
                    {
                        deck.Cards.Remove(this);
                        deck.Selected = null;
                    }
                    deck = value;
                    deck.Cards.Add(this);

                    Deck.UpdateCards();
                }
            }
        }

        private Deck subDeck = null;
        public Deck SubDeck
        {
            get
            {
                if (!HasSubDeck)
                {
                    subDeck = (Instantiate(Game.Instance.ingridientDeckPrefab) as GameObject).GetComponent<Deck>();
                    subDeck.order = deck.order-1;
                    subDeck.transform.parent = transform;
                    subDeck.transform.localPosition = Vector3.zero + new Vector3(subDeck.cardSpacerX, subDeck.cardSpacerY) * .4f;
                    //ingridientDeck.maxCardsSpace = 10;
                    //ingridientDeck.cardSpacerX = 0.05f;
                    //ingridientDeck.cardSpacerY = 0.25f;
                }
                
                return subDeck;
            }
        }
        public bool HasSubDeck { get { return subDeck != null; } }
        public void DiscardSubDeck (Deck discard)
        {
            if (subDeck == null)
                return;

            for (int q = 0; q < subDeck.Count; ++q)
                subDeck.Cards[q].Deck = discard;

            Destroy (subDeck);
            subDeck = null;
        }

        bool visible = true;
        /// <summary>
        /// True if the card is face up. False if the card back is showing.
        /// </summary>
        public bool Visible
        {
            get
            {
                return visible;
            }
            set
            {
                if (visible != value)
                {
                    visible = value;
                    //UpdateCardSprite();
                }
            }
        }

        bool isDragable = true;
        public bool IsDragable
        {
            get { return isDragable; }
            set { isDragable = value; }
        }

        /// <summary>
        /// Move this card to a random position in it's deck stack.
        /// </summary>
        public void Shuffle(System.Random random)
        {
            MoveToIndex(random.Next(0, Deck.Count));
        }

        /// <summary>
        /// Move this card to a specific position in it's deck stack.
        /// </summary>
        public void MoveToIndex(int index)
        {
            Deck.Cards.Remove(this);
            Deck.Cards.Insert(index, this);
        }

        bool isDragging = false;
        Vector3 clickPosition;
        float doubleClickStart = 0;
        bool dragDisabled;

        public void Start()
        {
            var particleSystem = GetComponentInChildren<ParticleSystem>();
            if (particleSystem != null)
                particleSystem.GetComponent<Renderer>().sortingLayerName = "Foreground"; // see http://answers.unity3d.com/questions/577288/particle-system-rendering-behind-sprites.html
        }


        void OnMouseEnter()
        {
            Game.Instance.OnMouseEnter(this, deck);
        }

        void OnMouseExit()
        {
            Game.Instance.OnMouseExit(this, deck);
        }

        void OnMouseDown()
        {
            Game.Instance.OnMouseDown(this, deck);
        }

        void OnMouseUp()
        {
        }

        /// <summary>
        /// Move this card to a specified position in a specified deck.
        /// </summary>
        public void SetDeck(int newPosition, Deck deck = null)
        {
            Deck.Cards.Remove(this);
            if (deck != null)
            {
                Deck.UpdateCards();
                this.deck = deck;
            }
            Deck.Cards.Insert(newPosition, this);
            Deck.UpdateCards();
        }
    }
